﻿/*** TABLES ***/

/* Producers table */
CREATE TABLE [Producers] (
	[ProducerID]	int				NOT NULL	IDENTITY(1, 1),
	[Name]			nvarchar(100)	NOT NULL,
	[Country]		nvarchar(100)	NULL,

	CONSTRAINT [PK_Producers] PRIMARY KEY ([ProducerID])
)
GO

/* Brands table */
CREATE TABLE [Brands] (
	[BrandID]		int				NOT NULL	IDENTITY(1, 1),
	[ProducerID]	int				NOT NULL,
	[Name]			nvarchar(100)	NOT NULL,
	[Description]	nvarchar(MAX)	NULL,

	CONSTRAINT [PK_Brands] PRIMARY KEY ([BrandID]),
	CONSTRAINT [FK_Brands_Producer] FOREIGN KEY ([ProducerID])
		REFERENCES [Producers] ([ProducerID]) ON DELETE CASCADE
)
GO

/* Products table */
CREATE TABLE [Products] (
	[ProductID]		int				NOT NULL	IDENTITY(1, 1),
	[BrandID]		int				NOT NULL,
	[Name]			nvarchar(100)	NOT NULL,
	[EnergyValue]	decimal(5, 1)	NOT NULL,
	[Proteins]		decimal(5, 1)	NULL,
	[Fats]			decimal(5, 1)	NULL,
	[Carbohydrates]	decimal(5, 1)	NULL,

	CONSTRAINT [PK_Products] PRIMARY KEY ([ProductID]),
	CONSTRAINT [FK_Products_Brand] FOREIGN KEY ([BrandID])
		REFERENCES [Brands] ([BrandID]) ON DELETE CASCADE
)
GO

/* Ingredients table */
CREATE TABLE [Ingredients] (
	[IngredientID]	int				NOT NULL	IDENTITY(1, 1),
	[Name]			nvarchar(100)	NOT NULL,

	CONSTRAINT [PK_Ingredients] PRIMARY KEY ([IngredientID])
)
GO

/* Product ingredients table */
CREATE TABLE [ProductIngredients] (
	[ProductID]		int				NOT NULL,
	[IngredientID]	int				NOT NULL,
	[Amount]		decimal(9, 6)	NULL,

	CONSTRAINT [PK_ProductIngredients] PRIMARY KEY ([ProductID], [IngredientID]),
	CONSTRAINT [FK_ProductIngredients_Product] FOREIGN KEY ([ProductID])
		REFERENCES [Products] ([ProductID]) ON DELETE CASCADE,
	CONSTRAINT [FK_ProductIngredients_Ingredient] FOREIGN KEY ([IngredientID])
		REFERENCES [Ingredients] ([IngredientID]) ON DELETE CASCADE
)
GO


/*** STORED PROCEDURES ***/

/* Producer creation procedure */
CREATE PROCEDURE [CreateProducer]
	@vName nvarchar(100), @vCountry nvarchar(100), @vProducerID int OUTPUT
AS
	INSERT INTO [Producers] ([Name], [Country])
	VALUES (@vName, @vCountry)

	SELECT @vProducerID = SCOPE_IDENTITY()

GO

/* Brand creation procedure */
CREATE PROCEDURE [CreateBrand]
	@vProducerID int, @vName nvarchar(100), @vDescription nvarchar(MAX),
	@vBrandID int OUTPUT
AS
	INSERT INTO [Brands] ([ProducerID], [Name], [Description])
	VALUES (@vProducerID, @vName, @vDescription)

	SELECT @vBrandID = SCOPE_IDENTITY()

GO

/* Product creation procedure */
CREATE PROCEDURE [CreateProduct]
	@vBrandID int, @vName nvarchar(100), @vEnergyValue decimal(5, 1),
	@vProteins decimal(5, 1), @vFats decimal(5, 1), @vCarbohydrates decimal(5, 1),
	@vProductID int OUTPUT
AS
	INSERT INTO [Products] ([BrandID], [Name], [EnergyValue], [Proteins],
							[Fats], [Carbohydrates])
	VALUES (@vBrandID, @vName, @vEnergyValue, @vProteins, @vFats, @vCarbohydrates)

	SELECT @vProductID = SCOPE_IDENTITY()

GO

/* Ingredient creation procedure */
CREATE PROCEDURE [CreateIngedient]
	@vName nvarchar(100), @vIngredientID int OUTPUT
AS
	INSERT INTO [Ingredients] ([Name]) VALUES (@vName)

	SELECT @vIngredientID = SCOPE_IDENTITY()

GO

/* Add product ingredient procedure */
CREATE PROCEDURE [AddProductIngredient]
	@vProductID int, @vIngredientID int, @vAmount decimal(9, 6)
AS
	IF (EXISTS(SELECT * FROM [ProductIngredients]
			   WHERE ([ProductID] = @vProductID)
				 AND ([IngredientID] = @vIngredientID)))
	BEGIN
		UPDATE [ProductIngredients]
		SET [Amount] = @vAmount
		WHERE ([ProductID] = @vProductID) AND ([IngredientID] = @vIngredientID)
	END
	ELSE
	BEGIN
		INSERT INTO [ProductIngredients] ([ProductID], [IngredientID], [Amount])
		VALUES (@vProductID, @vIngredientID, @vAmount)
	END

GO


/*** FILL INITIAL DATA ***/

/* Fill producers */
SET IDENTITY_INSERT [Producers] ON

INSERT INTO [Producers] ([ProducerID], [Name], [Country])
VALUES (1, N'Coca-Cola', N'България')
INSERT INTO [Producers] ([ProducerID], [Name], [Country])
VALUES (2, N'Kraft Foods', N'България')
INSERT INTO [Producers] ([ProducerID], [Name], [Country])
VALUES (3, N'Победа', N'България')
INSERT INTO [Producers] ([ProducerID], [Name], [Country])
VALUES (4, N'Обединена млечна компания', N'България')
INSERT INTO [Producers] ([ProducerID], [Name], [Country])
VALUES (5, N'Chocoladefabriken Lindt & Sprüngli', N'Швейцария')

SET IDENTITY_INSERT [Producers] OFF
GO

/* Fill brands */
SET IDENTITY_INSERT [Brands] ON

INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (1, 1, N'Coca-Cola', N'Винаги Coca-Cola!')
INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (2, 1, N'Fanta', N'Дръж се естествено')
INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (3, 1, N'Sprite', NULL)
INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (4, 1, N'Nestea', NULL)
INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (5, 2, N'Milka', N'Едно парченце и си там...')
INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (6, 2, N'Suchard', NULL)
INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (7, 3, N'Еверест', NULL)
INSERT INTO [Brands] ([BrandID], [ProducerID], [Name], [Description])
VALUES (8, 4, N'Верея', NULL)

SET IDENTITY_INSERT [Brands] OFF
GO

/* Fill products */
SET IDENTITY_INSERT [Products] ON

INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (1, 1, N'Coca-Cola', 45.1, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (2, 1, N'Coca-Cola Light', 0.4, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (3, 2, N'Fanta портокал', 47.3, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (4, 2, N'Fanta лимон', 42.2, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (5, 2, N'Fanta грейпфрут', 46.9, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (6, 3, N'Sprite', 47.8, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (7, 4, N'Nestea лимон', 30.2, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (8, 4, N'Nestea праскова', 29.5, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (9, 4, N'Nestea горски полодове', 32, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (10, 4, N'Nestea зелен чай и ягода', 31.7, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (11, 4, N'Nestea манго и ананас', 32, NULL, NULL, NULL)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (12, 5, N'Шоколад Milka цял лешник', 560, 8.4, 36, 50)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (13, 5, N'Шоколад Milka лешници и стафиди', 496, 6.4, 25.6, 58)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (14, 5, N'Шоколад Milka бял', 576, 5.2, 38.4, 52)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (15, 5, N'Бисквити Milka Джафа портокал', 360.4, 3.4, 6.8, 71.4)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (16, 7, N'Бисквити Еверест какао', 491, 7.4, 20.1, 70)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (17, 8, N'Кисело мляко Верея 4.5% масленост', 71.3, 3.5, 4.5, 4.2)
INSERT INTO [Products] ([ProductID], [BrandID], [Name], [EnergyValue], [Proteins], [Fats], [Carbohydrates])
VALUES (18, 8, N'Прясно мляко Верея 3% масленост', 55.4, 3, 3, 4.1)

SET IDENTITY_INSERT [Products] OFF
GO

/* Fill ingredients */
SET IDENTITY_INSERT [Ingredients] ON

INSERT INTO [Ingredients] ([IngredientID], [Name])
VALUES (1, N'Витамин C')
INSERT INTO [Ingredients] ([IngredientID], [Name])
VALUES (2, N'Натриев глутамат')
INSERT INTO [Ingredients] ([IngredientID], [Name])
VALUES (3, N'Хидрогенирани мазнини')
INSERT INTO [Ingredients] ([IngredientID], [Name])
VALUES (4, N'Лешници')
INSERT INTO [Ingredients] ([IngredientID], [Name])
VALUES (5, N'Натрий')

SET IDENTITY_INSERT [Ingredients] OFF
GO

/* Fill product ingredients */
INSERT INTO [ProductIngredients] ([ProductID], [IngredientID], [Amount])
VALUES (12, 4, NULL)
INSERT INTO [ProductIngredients] ([ProductID], [IngredientID], [Amount])
VALUES (13, 4, NULL)
INSERT INTO [ProductIngredients] ([ProductID], [IngredientID], [Amount])
VALUES (16, 3, NULL)
INSERT INTO [ProductIngredients] ([ProductID], [IngredientID], [Amount])
VALUES (7, 5, 0.012)

GO